# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 监控基类（纯净版，无任何 Django 依赖），可直接拷贝到第三方库使用
# @Time   : 2025-01-09
# @Author : 毛鹏

import atexit
import pymysql
from datetime import datetime
from abc import ABC, abstractmethod
from typing import Dict, Any, Optional


class MonitorBase(ABC):
    """
    监控基类，仅包含：
        - MySQL 连接管理
        - 基础日志输出（print）
    方便直接复制到第三方库（如 mangotools）中复用。
    """

    def __init__(self, mysql_config: Optional[Dict[str, Any]] = None, task_id: Optional[int] = None):
        """
        初始化监控基类

        Args:
            mysql_config: MySQL配置字典（可选），包含host, port, user, password, database等
                         如果不提供，则不初始化数据库连接
            task_id: 监控任务ID（可选），这里只是一个普通属性，方便你在脚本中自行使用
        """
        self.mysql_config = mysql_config
        self._db_connection = None
        self.task_id = task_id

        # 如果没有显式传入 task_id，尝试从命令行参数读取
        if self.task_id is None:
            import sys
            if len(sys.argv) > 1:
                try:
                    self.task_id = int(sys.argv[1])
                except (ValueError, IndexError):
                    pass

        if mysql_config:
            self._setup_mysql_connection()

    # ===== MySQL 相关 =====
    def _setup_mysql_connection(self):
        """设置MySQL连接"""
        if self._db_connection is None or (hasattr(self._db_connection, 'open') and not self._db_connection.open):
            self._db_connection = pymysql.connect(
                host=self.mysql_config['host'],
                port=self.mysql_config['port'],
                user=self.mysql_config['user'],
                password=self.mysql_config['password'],
                database=self.mysql_config['database'],
                charset='utf8mb4',
                cursorclass=pymysql.cursors.DictCursor,
                autocommit=True,  # 自动提交事务
            )
            self.log("数据库连接已创建")

        # 注册退出时关闭数据库连接
        atexit.register(self._close_db_connection)

    def _close_db_connection(self):
        """关闭数据库连接"""
        if self._db_connection and self._db_connection.open:
            self._db_connection.close()
            self.log("数据库连接已关闭")

    def ensure_connection(self):
        """确保数据库连接有效"""
        if not self.mysql_config:
            raise RuntimeError("MySQL配置未提供，无法确保数据库连接")
        if not self._db_connection or not self._db_connection.open:
            self.log("数据库连接已断开，重新连接...")
            self._setup_mysql_connection()

    def get_db_connection(self):
        """获取数据库连接"""
        if not self.mysql_config:
            raise RuntimeError("MySQL配置未提供，无法获取数据库连接")
        self.ensure_connection()
        return self._db_connection

    def execute_query(self, sql: str, params: tuple = None) -> Optional[Dict[str, Any]]:
        """
        执行数据库查询，返回单条记录
        """
        if not self.mysql_config:
            self.log_warning("MySQL配置未提供，跳过数据库查询")
            return None

        try:
            self.ensure_connection()
            with self._db_connection.cursor() as cursor:
                cursor.execute(sql, params or ())
                result = cursor.fetchone()
                return result
        except pymysql.Error as e:
            self.log_error(f"数据库查询错误: {e}")
            # 尝试重新连接一次
            try:
                self._setup_mysql_connection()
                with self._db_connection.cursor() as cursor:
                    cursor.execute(sql, params or ())
                    result = cursor.fetchone()
                    return result
            except Exception as retry_error:
                self.log_error(f"重试查询也失败: {retry_error}")
                return None
        except Exception as e:
            self.log_error(f"执行查询时发生其他错误: {e}")
            return None

    def execute_query_many(self, sql: str, params: tuple = None) -> list:
        """
        执行查询返回多条记录
        """
        if not self.mysql_config:
            self.log_warning("MySQL配置未提供，跳过数据库批量查询")
            return []

        try:
            self.ensure_connection()
            with self._db_connection.cursor() as cursor:
                cursor.execute(sql, params or ())
                results = cursor.fetchall()
                return results or []
        except pymysql.Error as e:
            self.log_error(f"数据库批量查询错误: {e}")
            return []
        except Exception as e:
            self.log_error(f"执行批量查询时发生其他错误: {e}")
            return []

    # ===== 日志相关（纯 print，不依赖任何框架） =====
    def log(self, message: str, level: str = "INFO"):
        """
        基础日志输出方法，自动添加时间戳

        这里不做任何数据库/框架写入，仅 print，方便在任意环境使用。
        """
        timestamp = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        log_text = f"[{timestamp}] [{level}] {message}"
        print(log_text)

    def log_error(self, message: str):
        """错误日志"""
        self.log(message, "ERROR")

    def log_warning(self, message: str):
        """警告日志"""
        self.log(message, "WARNING")

    def log_debug(self, message: str):
        """调试日志"""
        self.log(message, "DEBUG")

    def send(self, send_text: str, msg: str = None):
        """
        通知方法（纯文本版）

        在纯净版中，不做任何数据库 / 邮件 / 企业微信 等发送逻辑，
        仅统一以一条 ERROR 级别日志的形式打印出来，方便你在第三方库中自己扩展。
        """
        base_msg = msg if msg else "监控任务通知"
        self.log_error(f"{base_msg} | send_text={send_text}")

    # ===== 业务入口 =====
    @abstractmethod
    def run(self):
        """运行监控的主要逻辑，由子类实现"""
        pass

    def start_monitoring(self):
        """启动监控服务"""
        self.log("开始启动监控服务...")
        try:
            self.run()
        except KeyboardInterrupt:
            self.log_warning("收到中断信号，正在退出...")
        except Exception as e:
            self.log_error(f"监控服务异常退出: {e}")
        finally:
            # 只有在有MySQL配置时才尝试关闭连接
            if self.mysql_config:
                self._close_db_connection()
            self.log("监控服务已停止")





# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: MySQL 连接池，支持多线程环境
# @Time   : 2024-01-04 10:00
# @Author : 毛鹏
import threading
import time
from queue import Queue, Empty, Full
from typing import Optional, Any
from datetime import datetime, date

import pymysql
from pymysql.err import InternalError, OperationalError, ProgrammingError
from pymysql.cursors import DictCursor

from ..exceptions import MangoToolsError
from ..exceptions.error_msg import ERROR_MSG_0001, ERROR_MSG_0033, ERROR_MSG_0035, ERROR_MSG_0034
from ..models import MysqlConingModel


class MysqlConnectionPool:
    """MySQL 连接池类，支持多线程环境"""

    def __init__(
        self,
        mysql_config: MysqlConingModel,
        pool_size: int = 10,
        max_overflow: int = 20,
        timeout: float = 30.0,
        retry_attempts: int = 3,
        retry_delay: float = 0.5,
        is_c: bool = True,
        is_rud: bool = False
    ):
        """
        初始化连接池

        Args:
            mysql_config: MySQL 配置
            pool_size: 连接池大小
            max_overflow: 最大溢出连接数
            timeout: 获取连接超时时间(秒)
            retry_attempts: 重试次数
            retry_delay: 重试延迟(秒)
            is_c: 是否允许查询操作
            is_rud: 是否允许增删改操作
        """
        self.mysql_config = mysql_config
        self.pool_size = pool_size
        self.max_overflow = max_overflow
        self.timeout = timeout
        self.retry_attempts = retry_attempts
        self.retry_delay = retry_delay
        self.is_c = is_c
        self.is_rud = is_rud

        # 连接池状态
        self._pool: Queue = Queue(maxsize=pool_size)
        self._overflow_connections = set()
        self._overflow_count = 0
        self._created_connections = 0

        # 线程同步
        self._lock = threading.RLock()
        self._overflow_lock = threading.Lock()

        # 初始化连接池
        self._initialize_pool()

    def _initialize_pool(self):
        """初始化连接池"""
        for _ in range(self.pool_size):
            try:
                conn = self._create_connection()
                self._pool.put(conn)
                self._created_connections += 1
            except Exception as e:
                # 如果连接失败，继续创建其他连接
                continue

        if self._created_connections == 0:
            raise MangoToolsError(*ERROR_MSG_0001)

    def _create_connection(self):
        """创建新的数据库连接"""
        try:
            connection = pymysql.connect(
                host=self.mysql_config.host,
                port=self.mysql_config.port,
                user=self.mysql_config.user,
                password=self.mysql_config.password,
                database=self.mysql_config.database,
                autocommit=True,
                cursorclass=DictCursor,
                charset='utf8mb4',
                connect_timeout=30
            )
            return connection
        except OperationalError:
            raise MangoToolsError(*ERROR_MSG_0001)
        except InternalError:
            raise MangoToolsError(*ERROR_MSG_0033, value=(self.mysql_config.database,))

    def get_connection(self):
        """获取连接"""
        with self._lock:
            try:
                # 首先尝试从池中获取连接
                connection = self._pool.get(timeout=self.timeout)
                if self._is_connection_valid(connection):
                    return connection
                else:
                    # 连接无效，关闭并创建新连接
                    connection.close()
                    return self._create_connection()
            except Empty:
                # 池为空，尝试创建溢出连接
                return self._create_overflow_connection()

    def _create_overflow_connection(self):
        """创建溢出连接"""
        with self._overflow_lock:
            if self._overflow_count >= self.max_overflow:
                raise MangoToolsError("连接池已满，无法创建更多连接")

            try:
                connection = self._create_connection()
                self._overflow_connections.add(connection)
                self._overflow_count += 1
                return connection
            except Exception:
                raise MangoToolsError("无法创建溢出连接")

    def put_connection(self, connection):
        """归还连接到池中"""
        if connection is None:
            return

        with self._lock:
            try:
                if connection in self._overflow_connections:
                    # 这是溢出连接
                    with self._overflow_lock:
                        self._overflow_connections.remove(connection)
                        self._overflow_count -= 1
                    connection.close()
                else:
                    # 这是池连接
                    if self._is_connection_valid(connection):
                        try:
                            self._pool.put(connection, timeout=1.0)
                        except Full:
                            # 池已满，关闭连接
                            connection.close()
                    else:
                        # 连接无效，关闭并创建新连接
                        connection.close()
                        try:
                            new_conn = self._create_connection()
                            self._pool.put(new_conn, timeout=1.0)
                        except Full:
                            new_conn.close()
            except Exception:
                # 确保连接被关闭
                try:
                    connection.close()
                except Exception:
                    pass

    def _is_connection_valid(self, connection):
        """检查连接是否有效"""
        if not connection or not connection.open:
            return False

        try:
            # 执行简单查询检查连接
            with connection.cursor() as cursor:
                cursor.execute("SELECT 1")
            return True
        except Exception:
            return False

    def close_all(self):
        """关闭所有连接"""
        with self._lock:
            # 关闭池中的连接
            while not self._pool.empty():
                try:
                    conn = self._pool.get_nowait()
                    conn.close()
                except Exception:
                    pass

            # 关闭溢出连接
            with self._overflow_lock:
                for conn in self._overflow_connections.copy():
                    try:
                        conn.close()
                    except Exception:
                        pass
                self._overflow_connections.clear()
                self._overflow_count = 0

    def __enter__(self):
        """上下文管理器入口"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器出口"""
        self.close_all()

    def __del__(self):
        """析构函数"""
        self.close_all()

    def get_pool_status(self):
        """获取连接池状态"""
        with self._lock:
            with self._overflow_lock:
                return {
                    'pool_size': self.pool_size,
                    'created_connections': self._created_connections,
                    'available_connections': self._pool.qsize(),
                    'overflow_connections': self._overflow_count,
                    'total_connections': self._pool.qsize() + self._overflow_count
                }


class MysqlPoolConnect:
    """基于连接池的 MySQL 连接类，参照 MysqlConnect 设计"""

    def __init__(self, mysql_config: MysqlConingModel, pool_size: int = 10,
                 max_overflow: int = 20, is_c: bool = True, is_rud: bool = False):
        """
        初始化连接池连接类

        Args:
            mysql_config: MySQL 配置
            pool_size: 连接池大小
            max_overflow: 最大溢出连接数
            is_c: 是否允许查询操作
            is_rud: 是否允许增删改操作
        """
        self.mysql_config = mysql_config
        self.is_c = is_c
        self.is_rud = is_rud

        # 创建连接池
        self.pool = MysqlConnectionPool(
            mysql_config=mysql_config,
            pool_size=pool_size,
            max_overflow=max_overflow,
            is_c=is_c,
            is_rud=is_rud
        )

        self._local = threading.local()

    def __enter__(self):
        """支持上下文管理器"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器退出时归还连接"""
        self.close()

    def __del__(self):
        """析构函数"""
        self.close()
        self.pool.close_all()

    def close(self):
        """归还连接到池中"""
        if hasattr(self._local, 'connection') and self._local.connection:
            self.pool.put_connection(self._local.connection)
            self._local.connection = None

    def _get_connection(self):
        """获取线程本地连接"""
        if not hasattr(self._local, 'connection') or self._local.connection is None:
            self._local.connection = self.pool.get_connection()
        return self._local.connection

    def _convert_datetime_to_string(self, data):
        """递归将日期时间对象转换为字符串"""
        if isinstance(data, dict):
            return {key: self._convert_datetime_to_string(value) for key, value in data.items()}
        elif isinstance(data, list):
            return [self._convert_datetime_to_string(item) for item in data]
        elif isinstance(data, (datetime, date)):
            return data.strftime('%Y-%m-%d %H:%M:%S')
        else:
            return data

    def _execute_query(self, cursor, sql):
        """执行查询语句并处理结果"""
        cursor.execute(sql)
        result = cursor.fetchall()

        # 将日期时间转换为字符串
        if result:
            result = self._convert_datetime_to_string(result)

        return result

    def _execute_update(self, cursor, sql):
        """执行更新语句"""
        cursor.execute(sql)
        # autocommit=True 所以不需要手动commit
        return cursor.rowcount

    def condition_execute(self, sql: str) -> list[dict] | list | int | None:
        """条件执行SQL语句"""
        if not sql or not sql.strip():
            return None

        sql_upper = sql.strip().upper()
        is_select = (sql_upper.startswith(('SELECT', 'WITH', 'SHOW', 'DESC', 'DESCRIBE')) or
                     'SELECT' in sql_upper.split(None, 1)[0])

        if is_select:
            if self.is_c:
                return self.execute(sql)
            else:
                return None
        else:
            if self.is_rud:
                return self.execute(sql)
            else:
                return None

    def execute(self, sql: str) -> list[dict] | int | list:
        """执行SQL语句"""
        connection = self._get_connection()
        max_retries = self.pool.retry_attempts
        retry_delay = self.pool.retry_delay

        for attempt in range(max_retries):
            try:
                with connection.cursor() as cursor:
                    sql_upper = sql.strip().upper()
                    is_select = (sql_upper.startswith(('SELECT', 'WITH', 'SHOW', 'DESC', 'DESCRIBE')) or
                                 'SELECT' in sql_upper.split(None, 1)[0])

                    if is_select:
                        result = self._execute_query(cursor, sql)
                        return result
                    else:
                        result = self._execute_update(cursor, sql)
                        return result

            except ProgrammingError as e:
                raise MangoToolsError(*ERROR_MSG_0034, value=(sql, str(e)))
            except InternalError as e:
                raise MangoToolsError(*ERROR_MSG_0035, value=(str(e), ))
            except OperationalError as e:
                if "Lost connection" in str(e) or "MySQL server has gone away" in str(e):
                    if attempt < max_retries - 1:
                        # 重新获取连接
                        self.close()
                        time.sleep(retry_delay)
                        continue
                    else:
                        raise MangoToolsError(*ERROR_MSG_0034, value=(sql, str(e)))
                else:
                    raise MangoToolsError(*ERROR_MSG_0034, value=(sql, str(e)))
            except Exception as e:
                if attempt < max_retries - 1:
                    time.sleep(retry_delay)
                    continue
                else:
                    raise MangoToolsError(*ERROR_MSG_0034, value=(sql, str(e)))

    def get_pool_status(self):
        """获取连接池状态"""
        return self.pool.get_pool_status()
